package javax.swing.filechooser;

import java.util.ArrayList;
import java.io.IOException;
import java.awt.Image;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import java.io.FileNotFoundException;
import sun.awt.shell.ShellFolder;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.ref.WeakReference;
import javax.swing.UIManager;
import java.io.File;

public abstract class FileSystemView
{
    static FileSystemView windowsFileSystemView;
    static FileSystemView unixFileSystemView;
    static FileSystemView genericFileSystemView;
    private boolean useSystemExtensionHiding;
    
    public static FileSystemView getFileSystemView() {
        if (File.separatorChar == '\\') {
            if (FileSystemView.windowsFileSystemView == null) {
                FileSystemView.windowsFileSystemView = new WindowsFileSystemView();
            }
            return FileSystemView.windowsFileSystemView;
        }
        else if (File.separatorChar == '/') {
            if (FileSystemView.unixFileSystemView == null) {
                FileSystemView.unixFileSystemView = new UnixFileSystemView();
            }
            return FileSystemView.unixFileSystemView;
        }
        else {
            if (FileSystemView.genericFileSystemView == null) {
                FileSystemView.genericFileSystemView = new GenericFileSystemView();
            }
            return FileSystemView.genericFileSystemView;
        }
    }
    
    public FileSystemView() {
        this.useSystemExtensionHiding = UIManager.getDefaults().getBoolean("FileChooser.useSystemExtensionHiding");
        UIManager.addPropertyChangeListener(new PropertyChangeListener() {
            
            @Override
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                FileSystemView fileSystemView = (FileSystemView)weakReference.get();
                if (fileSystemView == null) {
                    UIManager.removePropertyChangeListener(this);
                }
                else if (propertyChangeEvent.getPropertyName().equals("lookAndFeel")) {
                    fileSystemView.useSystemExtensionHiding = UIManager.getDefaults().getBoolean("FileChooser.useSystemExtensionHiding");
                }
            }
        });
    }
    
    public boolean isRoot(File obj) {
        if (obj == null || !obj.isAbsolute()) {
            return false;
        }
        File[] roots = this.getRoots();
        int length = roots.length;
        int i = 0;
        while (i < length) {
            if (roots[i].equals(obj)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public Boolean isTraversable(File file) {
        return Boolean.valueOf(file.isDirectory());
    }
    
    public String getSystemDisplayName(File file) {
        if (file == null) {
            return null;
        }
        String s = file.getName();
        if (!s.equals("..") && !s.equals(".") && (this.useSystemExtensionHiding || !this.isFileSystem(file) || this.isFileSystemRoot(file))) {
            if (!(file instanceof ShellFolder) && !file.exists()) {
                return s;
            }
            try {
                s = this.getShellFolder(file).getDisplayName();
            }
            catch (FileNotFoundException ex) {
                return null;
            }
            if (s == null || s.length() == 0) {
                s = file.getPath();
            }
        }
        return s;
    }
    
    public String getSystemTypeDescription(File file) {
        return null;
    }
    
    public Icon getSystemIcon(File file) {
        if (file == null) {
            return null;
        }
        ShellFolder shellFolder;
        try {
            shellFolder = this.getShellFolder(file);
        }
        catch (FileNotFoundException ex) {
            return null;
        }
        Image icon = shellFolder.getIcon(false);
        if (icon != null) {
            return new ImageIcon(icon, shellFolder.getFolderType());
        }
        return UIManager.getIcon(file.isDirectory() ? "FileView.directoryIcon" : "FileView.fileIcon");
    }
    
    public boolean isParent(File obj, File file) {
        if (obj == null || file == null) {
            return false;
        }
        if ((obj instanceof ShellFolder)) {
            File parentFile = file.getParentFile();
            if (parentFile != null && parentFile.equals(obj)) {
                return true;
            }
            File[] files = this.getFiles(obj, false);
            int length = files.length;
            int i = 0;
            while (i < length) {
                if (file.equals(files[i])) {
                    return true;
                }
                ++i;
            }
            return false;
        }
        else {
            return obj.equals(file.getParentFile());
        }
    }
    
    public File getChild(File file, String anObject) {
        if ((file instanceof ShellFolder)) {
            File[] files = this.getFiles(file, false);
            int length = files.length;
            int i = 0;
            while (i < length) {
                File file2 = files[i];
                if (file2.getName().equals(anObject)) {
                    return file2;
                }
                ++i;
            }
        }
        return this.createFileObject(file, anObject);
    }
    
    public boolean isFileSystem(File file) {
        if ((file instanceof ShellFolder)) {
            ShellFolder shellFolder = (ShellFolder)file;
            return shellFolder.isFileSystem() && (!shellFolder.isLink() || !shellFolder.isDirectory());
        }
        return true;
    }
    
    public abstract File createNewFolder(File p0) throws IOException;
    
    public boolean isHiddenFile(File file) {
        return file.isHidden();
    }
    
    public boolean isFileSystemRoot(File dir) {
        return ShellFolder.isFileSystemRoot(dir);
    }
    
    public boolean isDrive(File file) {
        return false;
    }
    
    public boolean isFloppyDrive(File file) {
        return false;
    }
    
    public boolean isComputerNode(File dir) {
        return ShellFolder.isComputerNode(dir);
    }
    
    public File[] getRoots() {
        File[] array = (File[])(File[])ShellFolder.get("roots");
        for (int i = 0; i < array.length; ++i) {
            if (this.isFileSystemRoot(array[i])) {
                array[i] = this.createFileSystemRoot(array[i]);
            }
        }
        return array;
    }
    
    public File getHomeDirectory() {
        return this.createFileObject(System.getProperty("user.home"));
    }
    
    public File getDefaultDirectory() {
        File fileSystemRoot = (File)ShellFolder.get("fileChooserDefaultFolder");
        if (this.isFileSystemRoot(fileSystemRoot)) {
            fileSystemRoot = this.createFileSystemRoot(fileSystemRoot);
            return fileSystemRoot;
        }
        return fileSystemRoot;
    }
    
    public File createFileObject(File parent, String s) {
        if (parent == null) {
            return new File(s);
        }
        return new File(parent, s);
    }
    
    public File createFileObject(String pathname) {
        File fileSystemRoot = new File(pathname);
        if (this.isFileSystemRoot(fileSystemRoot)) {
            fileSystemRoot = this.createFileSystemRoot(fileSystemRoot);
            return fileSystemRoot;
        }
        return fileSystemRoot;
    }
    
    public File[] getFiles(File shellFolder, boolean b) {
        ArrayList<ShellFolder> list = new ArrayList();
        if (!(shellFolder instanceof ShellFolder)) {
            try {
                shellFolder = this.getShellFolder(shellFolder);
            }
            catch (FileNotFoundException ex) {
                return new File[0];
            }
        }
        File[] listFiles = ((ShellFolder)shellFolder).listFiles(!b);
        if (listFiles == null) {
            return new File[0];
        }
        int length = listFiles.length;
        int i = 0;
        while (i < length) {
            ShellFolder file = listFiles[i];
            if (Thread.currentThread().isInterrupted()) {
                break;
            }
            Label_0164: {
                if (!(file instanceof ShellFolder)) {
                    if (this.isFileSystemRoot(file)) {
                        file = this.createFileSystemRoot(file);
                    }
                    try {
                        file = ShellFolder.getShellFolder(file);
                    }
                    catch (FileNotFoundException ex2) {
                        break Label_0164;
                    }
                    catch (InternalError internalError) {
                        break Label_0164;
                    }
                }
                if (!b || !this.isHiddenFile(file)) {
                    list.add(file);
                }
            }
            ++i;
        }
        return (File[])list.toArray(new File[list.size()]);
    }
    
    public File getParentDirectory(File file) {
        if (file == null || !file.exists()) {
            return null;
        }
        ShellFolder shellFolder;
        try {
            shellFolder = this.getShellFolder(file);
        }
        catch (FileNotFoundException ex) {
            return null;
        }
        File file2 = shellFolder.getParentFile();
        if (file2 == null) {
            return null;
        }
        if (this.isFileSystem(file2)) {
            if (!file2.exists()) {
                File parentFile = file2.getParentFile();
                if (parentFile == null || !this.isFileSystem(parentFile)) {
                    file2 = this.createFileSystemRoot(file2);
                }
            }
            return file2;
        }
        else {
            return file2;
        }
    }
    
    ShellFolder getShellFolder(File fileSystemRoot) throws FileNotFoundException {
        if (!(fileSystemRoot instanceof ShellFolder) && !(fileSystemRoot instanceof FileSystemView.FileSystemRoot) && this.isFileSystemRoot(fileSystemRoot)) {
            fileSystemRoot = this.createFileSystemRoot(fileSystemRoot);
        }
        try {
            return ShellFolder.getShellFolder(fileSystemRoot);
        }
        catch (InternalError internalError) {
            System.err.println("FileSystemView.getShellFolder: f=" + fileSystemRoot);
            internalError.printStackTrace();
            return null;
        }
    }
    
    protected File createFileSystemRoot(File file) {
        return new FileSystemRoot(file);
    }
    
    static {
        FileSystemView.windowsFileSystemView = null;
        FileSystemView.unixFileSystemView = null;
        FileSystemView.genericFileSystemView = null;
    }
    
    static class FileSystemRoot extends File
    {
        public FileSystemRoot(File parent) {
            super(parent, "");
        }
        
        public FileSystemRoot(String pathname) {
            super(pathname);
        }
        
        @Override
        public boolean isDirectory() {
            return true;
        }
        
        @Override
        public String getName() {
            return this.getPath();
        }
    }
}
